@@ -4,7 +4,7 @@ import os
 import json
 
 from .config import Config
-from .group import Vmess, Vless, Socks, SS, Mtproto, Trojan
+from .group import Vmess, Vless, Socks, SS, Mtproto, Trojan, Xtls
 from .selector import ClientSelector
 
 class ClientWriter:
@@ -35,14 +35,16 @@ class ClientWriter:
             user_json["users"][0]["id"] = self.node.password
             user_json["users"][0]["alterId"] = self.node.alter_id
 
-        elif type(self.node) == Vless:
+        elif type(self.node) in (Vless, Xtls):
             self.client_config = self.load_template('client.json')
             user_json = self.client_config["outbounds"][0]["settings"]["vnext"][0]
             user_json["users"][0]["id"] = self.node.password
             del user_json["users"][0]["alterId"]
             del user_json["users"][0]["security"]
             user_json["users"][0]["encryption"] = self.node.encryption
-            self.client_config["outbounds"][0]["protocol"] = "vless"
+            if type(self.node) == Xtls:
+                user_json["users"][0]["flow"] = self.node.flow
+            self.client_config["outbounds"][0]["protocol"] = "vless" 
 
         elif type(self.node) == Socks:
             self.client_config = self.load_template('client_socks.json')
@@ -75,6 +77,11 @@ class ClientWriter:
 
         if self.group.tls == 'tls':
             self.client_config["outbounds"][0]["streamSettings"]["tlsSettings"] = {}
+        elif self.group.tls == 'xtls':
+            self.client_config["outbounds"][0]["streamSettings"]["xtlsSettings"]["serverName"] = self.group.ip
+            del self.client_config["outbounds"][0]["streamSettings"]["xtlsSettings"]["certificates"]
+            del self.client_config["outbounds"][0]["streamSettings"]["xtlsSettings"]["alpn"]
+            del self.client_config["outbounds"][0]["mux"]
 
     def write(self):
         '''
@@ -88,6 +95,8 @@ class ClientWriter:
 
 def generate():
     cs = ClientSelector(_('generate client json'))
+    if not hasattr(cs, 'client_index'):
+        return
     client_index = cs.client_index
     group = cs.group
 
